        list    p=12F629, r=dec
        #include <P12f629.inc>
       __CONFIG _INTRC_OSC_NOCLKOUT&_WDT_OFF&_CP_OFF&_MCLRE_OFF

;**************************************************************************
; Benefon Sigma Gold RF module controller
;
; 12.800 XCO
; 77.250 1st IF
;
; LMX2335 dual synthesizer
; Pic12F629
; 
; (c) Ramirez@phreedom.org
;**************************************************************************

;**************************************************************************
; 4MHz, 1instr. cycle = 1uS 
;
;**************************************************************************

;**************************************************************************
; I/O Ports
;
GP0		equ	0
GP1		equ	1
GP2		equ	2
GP3		equ	3
GP4		equ	4
GP5		equ	5

;**************************************************************************
; define
;
#define		CLK	GPIO,GP0
#define		Data	GPIO,GP1
#define		LE	GPIO,GP2

; PLL1	Rx
; PLL2	Tx
;
; Freq = N * (Fosc/R)   ;   (Fosc/R) = synth step (25kHz for example)
;
;
; N = (P*B)+A		P=64		B>=A		A<P 
; B (11bit) -> BL(8bit)  BH(3bit)
; A (7bit)
; 
; R (15bit) -> RL(8bit)  RH(7bit)
;
; Tx -> 17323 * 25 = 433075 = 433.075MHz
; Rx -> 20450 * 25 = 511250 = 511.250MHz ( - 77.250MHz 1stIF) = 434.000 MHz
;
#define		R1L	0x0			; R = 512d = 0x200
#define		R1H	0x2
							; N = 20450         N = (P*B)+A | P=64 | B>=A | A<P |
#define		N1A	34d			; A = 34
#define		N1BL	00111111b		; B = 319        
#define		N1BH	00000001b		;
#define		Rconf1	00011b	; Don't change!
#define		Nconf1	00b		; Don't change!

#define		R2L	0			; R = 512d = 0x200
#define		R2H	0x2
							; N = 17323
#define		N2A	43d			; A = 43
#define		N2BL	00001110b		; B = 270	    01 00001110b
#define		N2BH	00000001b		;
#define		Rconf2	00011b	; Don't change!
#define		Nconf2	00b		; Don't change!

#define		RF2R	00b			; Don't change!
#define		RF1R	10b			; Don't change!
#define		RF2N	01b			; Don't change!
#define		RF1N	11b			; Don't change!

;**************************************************************************
; Variables
;	Free memory at 0x20

count		equ	0x20
reg		equ	0x21
precount	equ	0x22

wait1		equ	0x23
wait2		equ	0x24

;**************************************************************************
; Vectors
;	
;	0x0 - init vector
;	0x4 - int vector
		org	0x0
		goto	init

;**************************************************************************
; Main program
;	Program memory at 0x5
		org	5
init
	bcf	status,rp0
	clrf	GPIO
	movlw	0x7
	movwf	CMCON		; GP<2:0> to digital IO
	bsf	status,rp0
	clrf	TRISIO		; GP<5:0> output	
	bcf	status,rp0

begin
	bcf	LE
	bcf	CLK

	call	wait50ms

; R1
	movlw	Rconf1
	movwf	reg
	movlw	5
	call	senddata

	movlw	R1H
	movwf	reg
	movlw	7
	call	senddata

	movlw	R1L
	movwf	reg
	movlw	8
	call	senddata

	movlw	RF1R
	movwf	reg
	movlw	2
	call	senddata

	bsf	LE
	bcf	LE

; R2
	movlw	Rconf2
	movwf	reg
	movlw	5
	call	senddata

	movlw	R2H
	movwf	reg
	movlw	7
	call	senddata

	movlw	R2L
	movwf	reg
	movlw	8
	call	senddata

	movlw	RF2R
	movwf	reg
	movlw	2
	call	senddata

	bsf	LE
	bcf	LE

; N1
	movlw	Nconf1
	movwf	reg
	movlw	2
	call	senddata

	movlw	N1BH
	movwf	reg
	movlw	3
	call	senddata

	movlw	N1BL
	movwf	reg
	movlw	8
	call	senddata

	movlw	N1A
	movwf	reg
	movlw	7
	call	senddata

	movlw	RF1N
	movwf	reg
	movlw	2
	call	senddata

	bsf	LE
	bcf	LE

; N2
	movlw	Nconf2
	movwf	reg
	movlw	2
	call	senddata

	movlw	N2BH
	movwf	reg
	movlw	3
	call	senddata

	movlw	N2BL
	movwf	reg
	movlw	8
	call	senddata

	movlw	N2A
	movwf	reg
	movlw	7
	call	senddata

	movlw	RF2N
	movwf	reg
	movlw	2
	call	senddata

	bsf	LE
	bcf	LE

	sleep

;**************************************************************************
; Send Data
;
; 	reg - data to send
;	w - bits to send (send last 'w' LSB bits)
senddata

		movwf	count			; bits to send
		movlw	8			; all bits
		movwf	precount

preloop		movfw	count
		subwf	precount,w
		btfsc	status,z
		goto	loop
		rlf	reg
		decf	precount
		goto	preloop

loop		rlf	reg
		btfsc	status,c
		bsf	Data
		btfss	status,c
		bcf	Data
		bsf	CLK
		bcf	CLK
		decfsz	count
		goto	loop

		return

;***************************************************************
; Wait 50mS
;
wait50ms
		movlw	0x80			; 1
		movwf	wait2			; 1

kill3a		movlw	0xff			; 1
		movwf	wait1			; 1
kill5a		decfsz	wait1			; 1(2)
		goto	kill5a			; 2

		decfsz	wait2			; 1(2)
		goto	kill3a			; 2

		return


		end
