;*******************************************************************
;
;   Routines for FSK 1200 bps modem with a PIC16C84
;   -----------------------------------------------
;
;   Free software by Rildo Pragana
;      e-mail:  rpragana@truenet.com.br
;      s-mail: Caixa Postal 7440 Recife 50,630-970 Brasil
;   partially based on software by Eric Smith, dtmf.asm (Thanks Eric)
;
;     Dedicated to all that do not spare computational power, by
; requesting more powerful machines and more memory, and in despite
; of this can make efficient devices to serve mankind.  It is
; for YOU, PIC programmer :-)
;   If you like PIC, Linux, and GNU software, you are not alone!
;
;Some notes:
;----------
;  This software sends a buffer full (with option for a single byte
;answer) each time, because of timing constraints.  You can use
;a simple protocol to exchange data between two similar devices,
;by framing each data chunk with STX, ETX, and answering with
;single byte messages (ACK, NAK).
;     If you need the (analog) circuits for the phne line, please
;ask me by e-mail.  I used this in a circuit for logging data from
;a PABX which sends to a "printer" each phone call done.
;   "connect" routine dial a phone number and waits for an answer
;(single byte ENQ).  FRSIZ lets you choose if the message is long
;or short.  You may change buffer size to suit your needs.
;  "tframe" frames a message and sends it (must be connected before).
;  "rxline" waits for a message arriving.
;  Notice that some variables are reused, but not at the same time.
;*******************************************************************
;
        LIST    p=16C84 ;target processor
        INCLUDE p16c84.inc
        __CONFIG 019h   ;CP off, PWRTE on, WDTE off, XT osc
        radix DEC
;
w       equ     0
f       equ     1
;
#define _c      STATUS,C
#define _z      STATUS,Z
#define _rp0    STATUS,RP0
#define _t0if   INTCON,2
#define _pwm    PORTA,3
#define _rxm    PORTA,4
#define _rly    PORTA,2
#define _rng    PORTB,2
#define ABRT    rflags,0
#define STOPB   rflags,1
#define STBIT   rflags,2
#define INOLD   rflags,3
#define SYNC    rflags,4
#define FRSIZ   rflags,5
#define BITO    rflags,6
#define RBIT    rflags,7
;
;<---------------------- rflags ------------------------->
;+------+------+------+------+------+------+------+------+
;| RBIT | BITO |FRSIZ | SYNC |INOLD |STBIT |STOPB | ABRT |
;+------+------+------+------+------+------+------+------+
; FRSIZ = 0 --> single byte , 1 --> full buffer
;
clockrate       equ     .6144000        ;crystal frequency
fclk            equ     clockrate/4     ;processor instruction rate
;-------------------------------------------------------------------
rflags  equ     0ch
trdet   equ     11h     ;time since last input transition
timout  equ     12h
;
count   equ     0dh     ;general use counter
delay   equ     0eh     ;delay counter
digit   equ     11h
delayms equ     12h     ;delay in ms
;
bitcnt  equ     0dh     ;bit counter
smpcnt  equ     0eh     ;sample counter
oldsmp  equ     0fh     ;last sample
dphase  equ     10h     ;phase difference
phase   equ     11h     ;current phase
sample  equ     12h     ;current sample
bytcnt  equ     13h     ;byte counter
;
buf     equ     17h
bufdat  equ     buf+3
buflim  equ     2ch
bcc     equ     buflim
tmp     equ     bcc     ;usado somente em pstore e psnum
tmcnt   equ     buf+1   ;usado como contador para armazenamento do buffer
;
        constant FULLBIT = 36
        constant HALFBIT = 17 ;FULLBIT/2
        constant STRTBIT = 58 ;(3 * FULLBIT)/2 + (1/8)FULLBIT = (13/8)FULLBIT
        constant HIWIN = 11
        constant SYNCBYTE = 56h ;sync byte
;
        constant PHMASK  = 0fh
        constant LODPH   = 1
        constant HIDPH   = 2
        constant ENQ = 05h
        constant STX = 02h
        constant ETX = 03h
        constant EOT = 04h
        constant SUB = 1ah
        constant ACK = 06h
        constant NAK = 15h
;
        org     H'2100'
phone   de      "0~2224023",0         ;phone number (dialing string)
;
        org 0
        goto start
;
msg     addwf   PCL,f
        dt      "FSK modem test",0
wave    addwf   PCL,f           ;2+2+2 (addwf+call+return)
        dt      20h,2ch,37h,3dh,3fh,3bh,32h,26h,19h,0dh,04h,00h,02h,08h,13h,20h
;===========================================================================
;
; Send single char message   === DON'T MOVE ===
;
putans  movwf   bufdat
        bcf     FRSIZ
;
;Prepare header,bcc and transmit buffer's message
;
tframe  call    ldcntf
        clrw
putbcc2 xorwf   INDF,w
        incf    FSR,f
        decfsz  tmcnt,f
        goto    putbcc2
        movwf   bcc
        movlw   buf
        movwf   FSR
        movlw   0ffh            ;PAD
        call    putch
        call    putch
        movlw   SYNCBYTE        ;synchronization char
        call    putch
;-------------------------------------------------------------
;       MODEM transmission
;-------------------------------------------------------------
txline  movlw   5               ;header+char+bcc
        btfsc   FRSIZ
        movlw   buflim-buf+1    ;total buffer's size
        movwf   bytcnt
        bcf     ABRT
        movlw   .15
        movwf   smpcnt
        movlw   .11
        movwf   bitcnt
        clrf    dphase
        movlw   20h
        movwf   phase
        clrf    sample
        bcf     _pwm
        movlw   buf             ;buffer's beginning (incl. header)
        movwf   FSR
xmblk   btfsc   ABRT            ;2
        return
        movf    sample,w        ;1
        movwf   oldsmp          ;1
        bsf     _pwm            ;1
        swapf   oldsmp,w        ;1
        andlw   3               ;1
        addwf   PCL,f           ;2+2
        goto    samp3
        goto    samp2
        goto    samp1
        goto    samp0
samp0   call    pulse           ;27
        call    nxsmp           ;16
        call    nxbit           ;16
        call    nxbyte          ;16
        goto    xmblk           ;2
samp1   call    nxsmp           ;16
        call    pulse           ;27
        call    nxbit           ;16
        call    nxbyte          ;16
        goto    xmblk           ;2
samp2   call    nxsmp           ;16
        call    nxbit           ;16
        call    pulse           ;27
        call    nxbyte          ;16
        goto    xmblk           ;2
samp3   call    nxsmp           ;16
        call    nxbit           ;16
        call    nxbyte          ;16
        call    pulse           ;27
        goto    xmblk           ;2
;
pulse   movf    oldsmp,w        ;1+2
        andlw   0fh             ;1
        addwf   PCL,f           ;2
        nop                     ;0 ~ 15
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        bcf     _pwm       ;1
        xorlw   0fh             ;1
        addwf   PCL,f           ;2
waste19 nop                     ;0 ~ 15
waste18 nop
waste17 nop
waste16 nop
waste15 nop
waste14 nop
waste13 nop
waste12 nop
waste11 nop
waste10 nop
waste9  nop
waste8  nop
waste7  nop
waste6  nop
waste5  nop
waste4  return                  ;2
;
nxsmp   movf    dphase,w        ;1+2 (call)
        addwf   phase,w         ;1
        andlw   PHMASK          ;1
        movwf   phase           ;1
        call    wave            ;6
        movwf   sample          ;1
        decf    smpcnt,f        ;1
        return                  ;2
;
nxbit   movf    smpcnt,f        ;1+2
        btfss   _z              ;2/1
        goto    nxbit1          ;2
        movlw   LODPH           ;1
        btfss   BITO            ;2
        movlw   HIDPH           ;
        movwf   dphase          ;1
        bsf     _c              ;1
        rrf     INDF,f          ;1
        bcf     BITO            ;1
        btfsc   _c              ;2
        bsf     BITO            ;
xreturn return                  ;2
nxbit1  incf    smpcnt,w        ;1+4
        btfss   _z              ;2/1
        goto    waste8          ;8
        movlw   .15             ;1 samples-1 for each cycle
        movwf   smpcnt          ;1
        decf    bitcnt,f        ;1
        goto    xreturn         ;3
;
nxbyte  movf    bitcnt,f        ;1+2
        btfss   _z              ;2/1
        goto    waste12         ;12
        incf    FSR,f           ;1 advance pointer
        movlw   .11             ;1
        movwf   bitcnt          ;1
        bcf     BITO            ;1 start bit
        decfsz  bytcnt,f        ;2/1
        goto    nxbyt1          ;2
        bsf     ABRT            ;1
nxbyt1  goto    xreturn         ;3

;-------------------------------------------------------------
;       MODEM reception
;-------------------------------------------------------------
rxans   bcf     FRSIZ           ;receive single char answer
rxline  movlw   2
        btfsc   FRSIZ
        movlw   buflim-bufdat+1
        movwf   bytcnt          ;number of bytes to receive
        clrf    trdet
        clrf    smpcnt
;
	bsf	RBIT
	movlw	11100001b
        andwf   rflags,f
        movlw   .8              ;bitcnt=8
        movwf   bitcnt
        call    fsrdat
        clrf    INDF            ;received byte=NULL (it may have garbage)
        call    waste8          ;may be used for other tasks
;
        ;movf    rclk1,w        ;uncomment this if you have a RTC (real-time clock)
                                ;that increments rclk1 each 8 seconds
        ;addlw   4               ;aprox. 32s time-out
        ;movwf   timout
;
        goto    pllsyn
;
pllend  incf    trdet,f         ;1+7
        movf    trdet,w         ;1
        sublw   HIWIN           ;1 ( HIWIN > trdet ) ==> Carry
        btfss   _c              ;2
        bsf     RBIT            
        movf    trdet,w         ;1
        sublw   FULLBIT         ;1 ( FULLBIT > trdet ) ==> Carry
        movlw   HALFBIT         ;1
        btfsc   _c              ;2/1
        goto    recbit          ;2
        movwf   trdet           ;1
;
        call    waste4          ; ...or use time-out if you have a
                                ; RTC (real-time clock) in sotware
        ;movf    timout,w        ;1
        ;subwf   rclk1,w         ;1
        ;btfsc   _c              ;2 if carry, rclk1 >= timout
        ;return
recbit  btfss   STBIT           ;2/1+17 waiting start bit?
        goto    recb1           ;2
        decfsz  smpcnt,f        ;2/1 time for another sample?
        goto    recb2           ;2
        movlw   FULLBIT         ;1  new bit window time
        movwf   smpcnt          ;1
        bcf     _c              ;1 carry=RBIT
        btfsc   RBIT            ;2
        bsf     _c
        rrf     INDF,f          ;1
        decfsz  bitcnt,f        ;2/1 counts one more received bit
        goto    recb3           ;2
        movlw   .8              ;1
        movwf   bitcnt          ;1
        bcf     STBIT           ;1 waits for another start bit (subtot=32)
        btfss   SYNC            ;2/1
        goto    recb4           ;2
        incf    FSR,f           ;1
        decfsz  bytcnt,f        ;2/1
        goto    $+2             ;2
        return
        goto    pllsyn          ;2
;
recb1   btfss   STOPB           ;2/1+20
        goto    recb1b          ;2
        btfss   RBIT            ;2/1
        goto    recb1a          ;2
        call    waste4          ;4
recb1c  goto    $+1             ;2
        call    waste8          ;8 may be used for other task
        goto    pllsyn          ;2
recb1b  btfsc   RBIT            ;2+23
        bsf     STOPB           ;
        call    waste8          ;8 may do something useful instead
        goto    $+1             ;2
        goto    pllsyn1         ;2
recb1a  bsf     STBIT           ;1+25
        bcf     STOPB           ;1
        movlw   STRTBIT         ;1
        movwf   smpcnt          ;1
        call    waste6          ;6
        goto    pllsyn1         ;2
recb2   call    waste12         ;12+23
        goto    pllsyn1          ;2
recb3   goto    $+1             ;2+32
        nop                     ;1
        goto    pllsyn1          ;2
pllsyn1 goto    pllsyn          ;2
recb4   movf    INDF,w          ;1+35
        sublw   SYNCBYTE        ;1
        btfsc   _z              ;2
        bsf     SYNC            ;1
pllsyn  btfss   _rxm            ;2/1
        goto    pll1            ;2
        nop                     ;1
        btfss   INOLD           ;2/1
        goto    plldifa         ;2
        goto    pllend          ;2
pll1    btfsc   INOLD           ;2/1+3
        goto    plldifb         ;2
        goto    pllend          ;2
plldifa bsf     INOLD           ;1
        goto    plldif          ;2
plldifb bcf     INOLD           ;1
        goto    plldif          ;2
plldif  movf    trdet,w         ;1+9
        sublw   HIWIN           ;1 ( HIWIN > trdet ) ==> Carry
        bcf     RBIT            ;1
        btfss   _c              ;2
        bsf     RBIT            ;
        clrf    trdet           ;1
        goto    recbit          ;2
;
; Get/Put char into the buffer
;
getch   movf    INDF,w
        goto    $+2             ;incrementa apontador e volta
putch   movwf   INDF
        incf    FSR,f
        return
;
; Check if BCC is correct -- (only messages with FRSIZ=1)
; returns _z set if BCC is right
;
chkbcc  call    ldcntf
chkb1   call    getch
        xorwf   bcc,f
        decfsz  tmcnt,f
        goto    chkb1
        movf    bcc,w
        return
;-------------------------------------------------------------
;       Dial Utilities
;-------------------------------------------------------------
dialnum bsf     _rly
        movlw   .20             ;delay 2.5s
        call    waitx
        movlw   LOW(phone)
        movwf   EEADR
dialnu1 call    emget
        andlw   03fh
        btfsc   _z
        return
        andlw   0fh     ;dial one digit
        btfsc   _z
        movlw   .10     ;dial 10 pulses if digit=0
        movwf   digit
        sublw   0eh     ;test if digit==0eh (delay) --> '~' == 0x7e
        btfsc   _z
        goto    digdly  ;only 1/2 second delay
digloop bcf     _rly    ;relay open (38ms)
        movlw   .38*2
        call    waitms
        bsf     _rly    ;relay close (62ms)
        movlw   .62*2
        call    waitms
        decfsz  digit,f
        goto    digloop
        movlw   .3      ;interdigit delay 300ms (nominal 375ms)
        call    waitx
digdly  movlw   .4      ;delay 1/2 s
        call    waitx
        goto    dialnu1
;
; Delay (W = 1/2 milliseconds units)
;
wait12  clrw                    ;delay 128ms
waitms  movwf   delayms
        clrf    delay
;
        call    waste10         ;change this if RTC (real-time clock)
        call    waste12         ;is implemented
        ;call    rtcupd
;
waitms1 decfsz  delay,f         ;256*3
        goto    waitms1
;
        call    waste10         ;see notes 6 lines above
        call    waste12
        ;call    rtcupd          ;22
;
        decfsz  delayms,f
        goto    waitms1
        return
;
; w * 1/8 second delay
;
wait1s  movlw   8
waitx   movwf   count
waitx1  movlw   .125*2
        call    waitms
        decfsz  count,f
        goto    waitx1
        return
;
; Connects and test received ENQ
;     returns carry=1 if error
;
connect call    dialnum
        call    rxans
        btfss   SYNC
        goto    disconn          ;sem resposta, erro
        movf    bufdat,w
        sublw   ENQ
        btfss   _z
        goto    disconn
retok   bcf     _c
        return
disconn bcf     _rly
reterr  bsf     _c
        return
;
;Loads FSR pointing to buffer's data area
;
ldcntf  call    ldtcnt
fsrdat  movlw   bufdat
        movwf   FSR
        return
;
;Loads counter with buffer's data area size
;
ldtcnt  movlw   buflim-bufdat
        movwf   tmcnt
        return
;
; Get a byte from EEPROM and advance pointer
;
emget   bsf     _rp0            ;get byte and advance pointer
        bsf     EECON1,RD
        bcf     _rp0
        movf    EEDATA,w
        incf    EEADR,f
        return
;-------------------------------------------------------------
;       INITIALIZATION
;-------------------------------------------------------------
start   movlw   00000001b
        movwf   PORTA
        movlw   10111000b
        movwf   PORTB
        clrf    rflags
        bsf     _rp0
        movlw   12h
        movwf   TRISA
        movlw   07h
        movwf   TRISB
        bcf     EECON1,EEIF
        movlw   10000111b       ;starts prescaler,timer0
        movwf   OPTION_REG
        bcf     _rp0
        bcf     _t0if
;
test    goto    test         ;======   put your code here   ======
;
;Make two PICs, one for transmitter test, the other for receiver test,
;using something like the routines below: (testorg and testans)
;
;You may wish to add a trigger signal for the oscilloscope, so add it
;just before 'call tframe' using an unassigned port.  At the receiving
;side you may add 2 leds (green and red) for telling if everything
;arrived. Turn them on at 'ansdone'(green) and 'anserr'(red). Don't
;forget to turn them off at the beginning!
;
testorg call    connect
        btfss   _c
        goto    orgerr
        clrf    count
        movlw   bufdat          ;fill data area with test message
        movwf   FSR
mvmem   movf    count,w
        call    msg
        andlw   0ffh            ;test for W = 0 (end-of-message)
        btfsc   _z
        goto    mvdone
        call    putch
        incf    count,f
        goto    mvmem
mvdone  call    tframe
        bcf     _rly            ;disconnect, all done
orgloop goto    orgloop         ;stop now
orgerr  goto    orgerr          ;did'nt connect?!
;------------------------------------------------------------------
testans btfss   _rng
        goto    answer
        goto    testans
answer  bsf     _rly            ;get "phone"
        call    wait1s          ;some time to get line ready (no noise)
        movlw   ENQ
        call    putans          ;send ENQ to the originating part
        bsf     FRSIZ           ;we'll receive now a full buffer
        call    rxline
        btfss   SYNC            ;if SYNC not found, perhaps a time-out occurred
        goto    anserr
        call    chkbcc          ;verifies if BCC is right for the msg received
        btfss   _z              ;zero flag set means it's ok
        goto    anserr
ansdone goto    ansdone         ;presto! The message received is in the buffer
                                ;(believe it or show it.) You may add some
                                ;simple RS-232 routines to send the buffer
                                ;out to a dumb terminal or PC
anserr  goto    anserr
;
     END
;

